@@ -1,16 +1,16 @@
-import datetime
 import json
-import requests
+import os
 import shutil
 import time
-import os
+
+import requests
 import yaml
 
 from sdcclient._common import _SdcCommon
+from sdcclient.secure import PolicyEventsClientV1, PolicyEventsClientOld
 
 
-class SdSecureClient(_SdcCommon):
-
+class SdSecureClient(PolicyEventsClientV1, PolicyEventsClientOld, _SdcCommon):
     def __init__(self, token="", sdc_url='https://secure.sysdig.com', ssl_verify=True, custom_headers=None):
         super(SdSecureClient, self).__init__(token, sdc_url, ssl_verify, custom_headers)
 
@@ -29,7 +29,8 @@ class SdSecureClient(_SdcCommon):
         return self._policy_v2
 
     def _get_falco_rules(self, kind):
-        res = requests.get(self.url + '/api/settings/falco/{}RulesFile'.format(kind), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.get(self.url + '/api/settings/falco/{}RulesFile'.format(kind), headers=self.hdrs,
+                           verify=self.ssl_verify)
         if not self._checkResponse(res):
             return [False, self.lasterr]
         data = res.json()
@@ -74,7 +75,8 @@ class SdSecureClient(_SdcCommon):
 
         payload[1]["{}RulesFile".format(kind)]["content"] = rules_content  # pylint: disable=unsubscriptable-object
 
-        res = requests.put(self.url + '/api/settings/falco/{}RulesFile'.format(kind), headers=self.hdrs, data=json.dumps(payload[1]), verify=self.ssl_verify)
+        res = requests.put(self.url + '/api/settings/falco/{}RulesFile'.format(kind), headers=self.hdrs,
+                           data=json.dumps(payload[1]), verify=self.ssl_verify)
         if not self._checkResponse(res):
             return [False, self.lasterr]
         return [True, res.json()]
@@ -114,7 +116,8 @@ class SdSecureClient(_SdcCommon):
     # Only one kind for now called "default", but might add a "custom" kind later.
     def _get_falco_rules_files(self, kind):
 
-        res = requests.get(self.url + '/api/settings/falco/{}RulesFiles'.format(kind), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.get(self.url + '/api/settings/falco/{}RulesFiles'.format(kind), headers=self.hdrs,
+                           verify=self.ssl_verify)
         if not self._checkResponse(res):
             return [False, self.lasterr]
         data = res.json()
@@ -276,7 +279,8 @@ class SdSecureClient(_SdcCommon):
         if "defaultPolicies" in rules_files:
             obj["defaultPolicies"] = rules_files["defaultPolicies"]
 
-        res = requests.put(self.url + '/api/settings/falco/{}RulesFiles'.format(kind), headers=self.hdrs, data=json.dumps(payload[1]), verify=self.ssl_verify)
+        res = requests.put(self.url + '/api/settings/falco/{}RulesFiles'.format(kind), headers=self.hdrs,
+                           data=json.dumps(payload[1]), verify=self.ssl_verify)
         if not self._checkResponse(res):
             return [False, self.lasterr]
         return [True, res.json()]
@@ -357,7 +361,7 @@ class SdSecureClient(_SdcCommon):
                     with open(cpath, 'r') as content_file:
                         try:
                             required_engine_version = int(os.path.basename(vpath))
-                            if vpath < 0:
+                            if int(os.path.basename(vpath)) < 0:
                                 return [False, "Variant directory {} must be a positive number".format(vpath)]
                             fobj["variants"].append({
                                 "requiredEngineVersion": required_engine_version,
@@ -372,203 +376,6 @@ class SdSecureClient(_SdcCommon):
 
         return [True, ret]
 
-    def _get_policy_events_int(self, ctx):
-        policy_events_url = self.url + '/api/policyEvents{id}?from={frm:d}&to={to:d}&offset={offset}&limit={limit}{sampling}{aggregations}{scope}{filter}'.format(
-            id="/%s" % ctx["id"] if "id" in ctx else "",
-            frm=int(ctx['from']),
-            to=int(ctx['to']),
-            offset=ctx['offset'],
-            limit=ctx['limit'],
-            sampling='&sampling=%d' % int(ctx['sampling']) if "sampling" in ctx else "",
-            aggregations='&aggregations=%s' % json.dumps(ctx['aggregations']) if "aggregations" in ctx else "",
-            scope='&scopeFilter=%s' % ctx['scopeFilter'] if "scopeFilter" in ctx else "",
-            filter='&eventFilter=%s' % ctx['eventFilter'] if "eventFilter" in ctx else "")
-
-        res = requests.get(policy_events_url, headers=self.hdrs, verify=self.ssl_verify)
-        if not self._checkResponse(res):
-            return [False, self.lasterr]
-
-        # Increment the offset by limit
-        ctx['offset'] += ctx['limit']
-
-        return [True, {"ctx": ctx, "data": res.json()}]
-
-    def get_policy_events_range(self, from_sec, to_sec, sampling=None, aggregations=None, scope_filter=None, event_filter=None):
-        '''**Description**
-            Fetch all policy events that occurred in the time range [from_sec:to_sec]. This method is used in conjunction
-            with :func:`~sdcclient.SdSecureClient.get_more_policy_events` to provide paginated access to policy events.
-
-        **Arguments**
-            - from_sec: the start of the timerange for which to get events
-            - end_sec: the end of the timerange for which to get events
-            - sampling: sample all policy events using *sampling* interval.
-            - aggregations: When present it specifies how to aggregate events (sampling does not need to be specified, because when it's present it automatically means events will be aggregated). This field can either be a list of scope metrics or a list of policyEvents fields but (currently) not a mix of the two. When policy events fields are specified, only these can be used= severity, agentId, containerId, policyId, ruleType.
-            - scope_filter: this is a SysdigMonitor-like filter (e.g 'container.image=ubuntu'). When provided, events are filtered by their scope, so only a subset will be returned (e.g. 'container.image=ubuntu' will provide only events that have happened on an ubuntu container).
-            - event_filter: this is a SysdigMonitor-like filter (e.g. policyEvent.policyId=3). When provided, events are filtered by some of their properties. Currently the supported set of filters is policyEvent.all(which can be used just with matches, policyEvent.policyId, policyEvent.id, policyEvent.severity, policyEvent.ruleTye, policyEvent.ruleSubtype.
-
-        **Success Return Value**
-            An array containing:
-              - A context object that should be passed to later calls to get_more_policy_events.
-              - An array of policy events, in JSON format. See :func:`~sdcclient.SdSecureClient.get_more_policy_events`
-                for details on the contents of policy events.
-
-        **Example**
-            `examples/get_secure_policy_events.py <https://github.com/draios/python-sdc-client/blob/master/examples/get_secure_policy_events.py>`_
-
-        '''
-        options = {"from": int(from_sec) * 1000000,
-                   "to": int(to_sec) * 1000000,
-                   "offset": 0,
-                   "limit": 1000,
-                   "sampling": sampling,
-                   "aggregations": aggregations,
-                   "scopeFilter": scope_filter,
-                   "eventFilter": event_filter}
-        ctx = {k: v for k, v in options.items() if v is not None}
-        return self._get_policy_events_int(ctx)
-
-    def get_policy_events_duration(self, duration_sec, sampling=None, aggregations=None, scope_filter=None, event_filter=None):
-        '''**Description**
-            Fetch all policy events that occurred in the last duration_sec seconds. This method is used in conjunction with
-            :func:`~sdcclient.SdSecureClient.get_more_policy_events` to provide paginated access to policy events.
-
-        **Arguments**
-            - duration_sec: Fetch all policy events that have occurred in the last *duration_sec* seconds.
-            - sampling: Sample all policy events using *sampling* interval.
-            - aggregations: When present it specifies how to aggregate events (sampling does not need to be specified, because when it's present it automatically means events will be aggregated). This field can either be a list of scope metrics or a list of policyEvents fields but (currently) not a mix of the two. When policy events fields are specified, only these can be used= severity, agentId, containerId, policyId, ruleType.
-            - scope_filter: this is a SysdigMonitor-like filter (e.g 'container.image=ubuntu'). When provided, events are filtered by their scope, so only a subset will be returned (e.g. 'container.image=ubuntu' will provide only events that have happened on an ubuntu container).
-            - event_filter: this is a SysdigMonitor-like filter (e.g. policyEvent.policyId=3). When provided, events are filtered by some of their properties. Currently the supported set of filters is policyEvent.all(which can be used just with matches, policyEvent.policyId, policyEvent.id, policyEvent.severity, policyEvent.ruleTye, policyEvent.ruleSubtype.
-
-        **Success Return Value**
-            An array containing:
-              - A context object that should be passed to later calls to get_more_policy_events.
-              - An array of policy events, in JSON format. See :func:`~sdcclient.SdSecureClient.get_more_policy_events`
-                for details on the contents of policy events.
-
-        **Example**
-            `examples/get_secure_policy_events.py <https://github.com/draios/python-sdc-client/blob/master/examples/get_secure_policy_events.py>`_
-
-        '''
-        epoch = datetime.datetime.utcfromtimestamp(0)
-        to_ts = (datetime.datetime.utcnow() - epoch).total_seconds() * 1000 * 1000
-        from_ts = to_ts - (int(duration_sec) * 1000 * 1000)
-
-        options = {"to": to_ts,
-                   "from": from_ts,
-                   "offset": 0,
-                   "limit": 1000,
-                   "sampling": sampling,
-                   "aggregations": aggregations,
-                   "scopeFilter": scope_filter,
-                   "eventFilter": event_filter}
-        ctx = {k: v for k, v in options.items() if v is not None}
-        return self._get_policy_events_int(ctx)
-
-    def get_policy_events_id_range(self, id, from_sec, to_sec, sampling=None, aggregations=None, scope_filter=None, event_filter=None):
-        '''**Description**
-            Fetch all policy events with id that occurred in the time range [from_sec:to_sec]. This method is used in conjunction
-            with :func:`~sdcclient.SdSecureClient.get_more_policy_events` to provide paginated access to policy events.
-
-        **Arguments**
-            - id: the id of the policy events to fetch.
-            - from_sec: the start of the timerange for which to get events
-            - end_sec: the end of the timerange for which to get events
-            - sampling: sample all policy events using *sampling* interval.
-            - scope_filter: this is a SysdigMonitor-like filter (e.g 'container.image=ubuntu'). When provided, events are filtered by their scope, so only a subset will be returned (e.g. 'container.image=ubuntu' will provide only events that have happened on an ubuntu container).
-            - event_filter: this is a SysdigMonitor-like filter (e.g. policyEvent.policyId=3). When provided, events are filtered by some of their properties. Currently the supported set of filters is policyEvent.all(which can be used just with matches, policyEvent.policyId, policyEvent.id, policyEvent.severity, policyEvent.ruleTye, policyEvent.ruleSubtype.
-            - aggregations: When present it specifies how to aggregate events (sampling does not need to be specified, because when it's present it automatically means events will be aggregated). This field can either be a list of scope metrics or a list of policyEvents fields but (currently) not a mix of the two. When policy events fields are specified, only these can be used= severity, agentId, containerId, policyId, ruleType.
-
-        **Success Return Value**
-            An array containing:
-              - A context object that should be passed to later calls to get_more_policy_events.
-              - An array of policy events, in JSON format. See :func:`~sdcclient.SdSecureClient.get_more_policy_events`
-                for details on the contents of policy events.
-
-        **Example**
-            `examples/get_secure_policy_events.py <https://github.com/draios/python-sdc-client/blob/master/examples/get_secure_policy_events.py>`_
-
-        '''
-        options = {"id": id,
-                   "from": int(from_sec) * 1000000,
-                   "to": int(to_sec) * 1000000,
-                   "offset": 0,
-                   "limit": 1000,
-                   "sampling": sampling,
-                   "aggregations": aggregations,
-                   "scopeFilter": scope_filter,
-                   "eventFilter": event_filter}
-        ctx = {k: v for k, v in options.items() if v is not None}
-        return self._get_policy_events_int(ctx)
-
-    def get_policy_events_id_duration(self, id, duration_sec, sampling=None, aggregations=None, scope_filter=None, event_filter=None):
-        '''**Description**
-            Fetch all policy events with id that occurred in the last duration_sec seconds. This method is used in conjunction with
-            :func:`~sdcclient.SdSecureClient.get_more_policy_events` to provide paginated access to policy events.
-
-        **Arguments**
-            - id: the id of the policy events to fetch.
-            - duration_sec: Fetch all policy events that have occurred in the last *duration_sec* seconds.
-            - sampling: Sample all policy events using *sampling* interval.
-            - aggregations: When present it specifies how to aggregate events (sampling does not need to be specified, because when it's present it automatically means events will be aggregated). This field can either be a list of scope metrics or a list of policyEvents fields but (currently) not a mix of the two. When policy events fields are specified, only these can be used= severity, agentId, containerId, policyId, ruleType.
-            - scope_filter: this is a SysdigMonitor-like filter (e.g 'container.image=ubuntu'). When provided, events are filtered by their scope, so only a subset will be returned (e.g. 'container.image=ubuntu' will provide only events that have happened on an ubuntu container).
-            - event_filter: this is a SysdigMonitor-like filter (e.g. policyEvent.policyId=3). When provided, events are filtered by some of their properties. Currently the supported set of filters is policyEvent.all(which can be used just with matches, policyEvent.policyId, policyEvent.id, policyEvent.severity, policyEvent.ruleTye, policyEvent.ruleSubtype.
-
-        **Success Return Value**
-            An array containing:
-              - A context object that should be passed to later calls to get_more_policy_events.
-              - An array of policy events, in JSON format. See :func:`~sdcclient.SdSecureClient.get_more_policy_events`
-                for details on the contents of policy events.
-
-        **Example**
-            `examples/get_secure_policy_events.py <https://github.com/draios/python-sdc-client/blob/master/examples/get_secure_policy_events.py>`_
-
-        '''
-        epoch = datetime.datetime.utcfromtimestamp(0)
-        to_ts = (datetime.datetime.utcnow() - epoch).total_seconds() * 1000 * 1000
-        from_ts = to_ts - (int(duration_sec) * 1000 * 1000)
-
-        options = {"id": id,
-                   "to": to_ts,
-                   "from": from_ts,
-                   "offset": 0,
-                   "limit": 1000,
-                   "sampling": sampling,
-                   "aggregations": aggregations,
-                   "scopeFilter": scope_filter,
-                   "eventFilter": event_filter}
-        ctx = {k: v for k, v in options.items() if v is not None}
-        return self._get_policy_events_int(ctx)
-
-    def get_more_policy_events(self, ctx):
-        '''**Description**
-            Fetch additional policy events after an initial call to :func:`~sdcclient.SdSecureClient.get_policy_events_range` /
-            :func:`~sdcclient.SdSecureClient.get_policy_events_duration` or a prior call to get_more_policy_events.
-
-        **Arguments**
-            - ctx: a context object returned from an initial call to :func:`~sdcclient.SdSecureClient.get_policy_events_range` /
-              :func:`~sdcclient.SdSecureClient.get_policy_events_duration` or a prior call to get_more_policy_events.
-
-        **Success Return Value**
-            An array containing:
-              - A context object that should be passed to later calls to get_more_policy_events()
-              - An array of policy events, in JSON format. Each policy event contains the following:
-                 - hostMac: the mac address of the machine where the event occurred
-                 - severity: a severity level from 1-7
-                 - timestamp: when the event occurred (ns since the epoch)
-                 - version: a version number for this message (currently 1)
-                 - policyId: a reference to the policy that generated this policy event
-                 - output: A string describing the event that occurred
-                 - id: a unique identifier for this policy event
-                 - isAggregated: if true, this is a combination of multiple policy events
-                 - containerId: the container in which the policy event occurred
-
-            When the number of policy events returned is 0, there are no remaining events and you can stop calling get_more_policy_events().
-
-        **Example**
-            `examples/get_secure_policy_events.py <https://github.com/draios/python-sdc-client/blob/master/examples/get_secure_policy_events.py>`_
-        '''
-        return self._get_policy_events_int(ctx)
-
     def create_default_policies(self):
         '''**Description**
             Create new policies based on the currently available set of rules. For now, this only covers Falco rules, but we might extend
@@ -672,7 +479,8 @@ class SdSecureClient(_SdcCommon):
         res = requests.get(self.url + '/api/v2/policies/{}'.format(id), headers=self.hdrs, verify=self.ssl_verify)
         return self._request_result(res)
 
-    def add_policy(self, name, description, rule_names=[], actions=[], scope=None, severity=0, enabled=True, notification_channels=[]):
+    def add_policy(self, name, description, rule_names=[], actions=[], scope=None, severity=0, enabled=True,
+                   notification_channels=[]):
         '''**Description**
             Add a new policy.
 
@@ -699,7 +507,8 @@ class SdSecureClient(_SdcCommon):
             "enabled": enabled,
             "notificationChannelIds": notification_channels
         }
-        res = requests.post(self.url + '/api/v2/policies', headers=self.hdrs, data=json.dumps(policy), verify=self.ssl_verify)
+        res = requests.post(self.url + '/api/v2/policies', headers=self.hdrs, data=json.dumps(policy),
+                            verify=self.ssl_verify)
         return self._request_result(res)
 
     def add_policy_json(self, policy_json):
@@ -719,10 +528,13 @@ class SdSecureClient(_SdcCommon):
 
         try:
             policy_obj = json.loads(policy_json)
+            if "origin" in policy_obj:
+                del policy_obj["origin"]
         except Exception as e:
             return [False, "policy json is not valid json: {}".format(str(e))]
 
-        res = requests.post(self.url + '/api/v2/policies', headers=self.hdrs, data=json.dumps(policy_obj), verify=self.ssl_verify)
+        res = requests.post(self.url + '/api/v2/policies', headers=self.hdrs, data=json.dumps(policy_obj),
+                            verify=self.ssl_verify)
         return self._request_result(res)
 
     def update_policy(self, id, name=None, description=None, rule_names=None, actions=None, scope=None,
@@ -766,7 +578,8 @@ class SdSecureClient(_SdcCommon):
         if notification_channels is not None:
             policy["notificationChannelIds"] = notification_channels
 
-        res = requests.put(self.url + '/api/v2/policies/{}'.format(id), headers=self.hdrs, data=json.dumps(policy), verify=self.ssl_verify)
+        res = requests.put(self.url + '/api/v2/policies/{}'.format(id), headers=self.hdrs, data=json.dumps(policy),
+                           verify=self.ssl_verify)
         return self._request_result(res)
 
     def update_policy_json(self, policy_json):
@@ -786,13 +599,16 @@ class SdSecureClient(_SdcCommon):
         '''
         try:
             policy_obj = json.loads(policy_json)
+            if "origin" in policy_obj:
+                del policy_obj["origin"]
         except Exception as e:
             return [False, "policy json is not valid json: {}".format(str(e))]
 
         if "id" not in policy_obj:
             return [False, "Policy Json does not have an 'id' field"]
 
-        res = requests.put(self.url + '/api/v2/policies/{}'.format(policy_obj["id"]), headers=self.hdrs, data=json.dumps(policy_obj), verify=self.ssl_verify)
+        res = requests.put(self.url + '/api/v2/policies/{}'.format(policy_obj["id"]), headers=self.hdrs,
+                           data=json.dumps(policy_obj), verify=self.ssl_verify)
         return self._request_result(res)
 
     def delete_policy_name(self, name):
@@ -864,7 +680,8 @@ class SdSecureClient(_SdcCommon):
         **Success Return Value**
             A JSON object representing the list of rules.
         '''
-        res = requests.get(self.url + '/api/secure/rules/groups?name={}'.format(name), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.get(self.url + '/api/secure/rules/groups?name={}'.format(name), headers=self.hdrs,
+                           verify=self.ssl_verify)
         return self._request_result(res)
 
     def get_rule_id(self, id):
@@ -899,7 +716,8 @@ class SdSecureClient(_SdcCommon):
             "details": details,
             "tags": tags
         }
-        res = requests.post(self.url + '/api/secure/rules', data=json.dumps(rule), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.post(self.url + '/api/secure/rules', data=json.dumps(rule), headers=self.hdrs,
+                            verify=self.ssl_verify)
         return self._request_result(res)
 
     def update_rule(self, id, details={}, description="", tags=[]):
@@ -926,7 +744,8 @@ class SdSecureClient(_SdcCommon):
             rule['description'] = description
         if tags:
             rule['tags'] = tags
-        res = requests.put(self.url + '/api/secure/rules/{}'.format(id), data=json.dumps(rule), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.put(self.url + '/api/secure/rules/{}'.format(id), data=json.dumps(rule), headers=self.hdrs,
+                           verify=self.ssl_verify)
         return self._request_result(res)
 
     def delete_rule(self, id):
@@ -969,7 +788,8 @@ class SdSecureClient(_SdcCommon):
         **Success Return Value**
             A JSON object representing the list of falco macros.
         '''
-        res = requests.get(self.url + '/api/secure/falco/macros/groups?name={}'.format(name), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.get(self.url + '/api/secure/falco/macros/groups?name={}'.format(name), headers=self.hdrs,
+                           verify=self.ssl_verify)
         return self._request_result(res)
 
     def get_falco_macro_id(self, id):
@@ -982,10 +802,11 @@ class SdSecureClient(_SdcCommon):
         **Success Return Value**
             A JSON object representing the falco macro.
         '''
-        res = requests.get(self.url + '/api/secure/falco/macros/{}'.format(id), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.get(self.url + '/api/secure/falco/macros/{}'.format(id), headers=self.hdrs,
+                           verify=self.ssl_verify)
         return self._request_result(res)
 
-    def add_falco_macro(self, name, condition):
+    def add_falco_macro(self, name, condition, append=False):
         '''**Description**
             Create a new macro
 
@@ -1001,9 +822,11 @@ class SdSecureClient(_SdcCommon):
             "condition": {
                 "components": [],
                 "condition": condition
-            }
+            },
+            "append": append
         }
-        res = requests.post(self.url + '/api/secure/falco/macros', data=json.dumps(macro), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.post(self.url + '/api/secure/falco/macros', data=json.dumps(macro), headers=self.hdrs,
+                            verify=self.ssl_verify)
         return self._request_result(res)
 
     def update_falco_macro(self, id, condition):
@@ -1023,7 +846,8 @@ class SdSecureClient(_SdcCommon):
         macro = res
         macro['condition']['condition'] = condition
 
-        res = requests.put(self.url + '/api/secure/falco/macros/{}'.format(id), data=json.dumps(macro), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.put(self.url + '/api/secure/falco/macros/{}'.format(id), data=json.dumps(macro),
+                           headers=self.hdrs, verify=self.ssl_verify)
         return self._request_result(res)
 
     def delete_falco_macro(self, id):
@@ -1036,7 +860,8 @@ class SdSecureClient(_SdcCommon):
         **Success Return Value**
             A JSON object representing the macro.
         '''
-        res = requests.delete(self.url + '/api/secure/falco/macros/{}'.format(id), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.delete(self.url + '/api/secure/falco/macros/{}'.format(id), headers=self.hdrs,
+                              verify=self.ssl_verify)
         return self._request_result(res)
 
     def list_falco_lists(self):
@@ -1066,7 +891,8 @@ class SdSecureClient(_SdcCommon):
         **Success Return Value**
             A JSON object representing the list of falco lists.
         '''
-        res = requests.get(self.url + '/api/secure/falco/lists/groups?name={}'.format(name), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.get(self.url + '/api/secure/falco/lists/groups?name={}'.format(name), headers=self.hdrs,
+                           verify=self.ssl_verify)
         return self._request_result(res)
 
     def get_falco_list_id(self, id):
@@ -1079,10 +905,11 @@ class SdSecureClient(_SdcCommon):
         **Success Return Value**
             A JSON object representing the falco list.
         '''
-        res = requests.get(self.url + '/api/secure/falco/lists/{}'.format(id), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.get(self.url + '/api/secure/falco/lists/{}'.format(id), headers=self.hdrs,
+                           verify=self.ssl_verify)
         return self._request_result(res)
 
-    def add_falco_list(self, name, items):
+    def add_falco_list(self, name, items, append=False):
         '''**Description**
             Create a new list
 
@@ -1097,9 +924,11 @@ class SdSecureClient(_SdcCommon):
             "name": name,
             "items": {
                 "items": items
-            }
+            },
+            "append": append
         }
-        res = requests.post(self.url + '/api/secure/falco/lists', data=json.dumps(flist), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.post(self.url + '/api/secure/falco/lists', data=json.dumps(flist), headers=self.hdrs,
+                            verify=self.ssl_verify)
         return self._request_result(res)
 
     def update_falco_list(self, id, items):
@@ -1119,7 +948,8 @@ class SdSecureClient(_SdcCommon):
         flist = res
         flist['items']['items'] = items
 
-        res = requests.put(self.url + '/api/secure/falco/lists/{}'.format(id), data=json.dumps(flist), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.put(self.url + '/api/secure/falco/lists/{}'.format(id), data=json.dumps(flist),
+                           headers=self.hdrs, verify=self.ssl_verify)
         return self._request_result(res)
 
     def delete_falco_list(self, id):
@@ -1132,10 +962,12 @@ class SdSecureClient(_SdcCommon):
         **Success Return Value**
             A JSON object representing the list.
         '''
-        res = requests.delete(self.url + '/api/secure/falco/lists/{}'.format(id), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.delete(self.url + '/api/secure/falco/lists/{}'.format(id), headers=self.hdrs,
+                              verify=self.ssl_verify)
         return self._request_result(res)
 
-    def add_compliance_task(self, name, module_name='docker-bench-security', schedule='06:00:00Z/PT12H', scope=None, enabled=True):
+    def add_compliance_task(self, name, module_name='docker-bench-security', schedule='06:00:00Z/PT12H', scope=None,
+                            enabled=True):
         '''**Description**
             Add a new compliance task.
 
@@ -1157,7 +989,8 @@ class SdSecureClient(_SdcCommon):
             "scope": scope,
             "schedule": schedule
         }
-        res = requests.post(self.url + '/api/complianceTasks', data=json.dumps(task), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.post(self.url + '/api/complianceTasks', data=json.dumps(task), headers=self.hdrs,
+                            verify=self.ssl_verify)
         return self._request_result(res)
 
     def list_compliance_tasks(self):
@@ -1214,7 +1047,8 @@ class SdSecureClient(_SdcCommon):
             'enabled': enabled
         }
         task.update({k: v for k, v in options.items() if v is not None})
-        res = requests.put(self.url + '/api/complianceTasks/{}'.format(id), data=json.dumps(task), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.put(self.url + '/api/complianceTasks/{}'.format(id), data=json.dumps(task), headers=self.hdrs,
+                           verify=self.ssl_verify)
         return self._request_result(res)
 
     def delete_compliance_task(self, id):
@@ -1224,7 +1058,8 @@ class SdSecureClient(_SdcCommon):
         **Arguments**
             - id: the id of the compliance task to delete
         '''
-        res = requests.delete(self.url + '/api/complianceTasks/{}'.format(id), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.delete(self.url + '/api/complianceTasks/{}'.format(id), headers=self.hdrs,
+                              verify=self.ssl_verify)
         if not self._checkResponse(res):
             return False, self.lasterr
 
@@ -1275,13 +1110,15 @@ class SdSecureClient(_SdcCommon):
         **Success Return Value**
             A CSV representation of the compliance task run result.
         '''
-        res = requests.get(self.url + '/api/complianceResults/{}/csv'.format(id), headers=self.hdrs, verify=self.ssl_verify)
+        res = requests.get(self.url + '/api/complianceResults/{}/csv'.format(id), headers=self.hdrs,
+                           verify=self.ssl_verify)
         if not self._checkResponse(res):
             return False, self.lasterr
 
         return True, res.text
 
-    def list_commands_audit(self, from_sec=None, to_sec=None, scope_filter=None, command_filter=None, limit=100, offset=0, metrics=[]):
+    def list_commands_audit(self, from_sec=None, to_sec=None, scope_filter=None, command_filter=None, limit=100,
+                            offset=0, metrics=[]):
         '''**Description**
             List the commands audit.
 
@@ -1305,8 +1142,8 @@ class SdSecureClient(_SdcCommon):
             url=self.url,
             offset=offset,
             limit=limit,
-            frm=int(from_sec * 10**6),
-            to=int(to_sec * 10**6),
+            frm=int(from_sec * 10 ** 6),
+            to=int(to_sec * 10 ** 6),
             scope="&scopeFilter=" + scope_filter if scope_filter else "",
             commandFilter="&commandFilter=" + command_filter if command_filter else "",
             metrics="&metrics=" + json.dumps(metrics) if metrics else "")
@@ -1326,7 +1163,7 @@ class SdSecureClient(_SdcCommon):
         url = "{url}/api/commands/{id}?from=0&to={to}{metrics}".format(
             url=self.url,
             id=id,
-            to=int(time.time() * 10**6),
+            to=int(time.time() * 10 ** 6),
             metrics="&metrics=" + json.dumps(metrics) if metrics else "")
         res = requests.get(url, headers=self.hdrs, verify=self.ssl_verify)
         return self._request_result(res)
@@ -1343,13 +1180,12 @@ class SdSecureClient(_SdcCommon):
 
         '''
         url = "{url}/api/v1/profiling/profileGroups/0/profiles".format(
-            url = self.url
+            url=self.url
         )
 
         res = requests.get(url, headers=self.hdrs, verify=self.ssl_verify)
         return self._request_result(res)
 
-
     def get_image_profile(self, profileId):
         '''**Description**
             Find the image profile with a (partial) profile ID <profileId> and return its json description.
@@ -1372,7 +1208,6 @@ class SdSecureClient(_SdcCommon):
         if not ok:
             return [False, self.lasterr]
 
-        
         '''
         The content of the json stored in the image_profiles dictionary:
 
@@ -1385,21 +1220,21 @@ class SdSecureClient(_SdcCommon):
             ]
         }
         '''
-        
+
         matched_profiles = self.__get_matched_profileIDs(profileId, image_profiles['profiles'])
-        
+
         # Profile ID not found
         if len(matched_profiles) == 0:
             return [False, "No profile with ID {}".format(profileId)]
-        
+
         # Principal workflow. Profile ID found
         elif len(matched_profiles) == 1:
             # Matched id. Return information
             url = "{url}/api/v1/profiling/profiles/{profileId}".format(
-                url = self.url,
-                profileId = matched_profiles[0]['profileId']
+                url=self.url,
+                profileId=matched_profiles[0]['profileId']
             )
-            
+
             res = requests.get(url, headers=self.hdrs, verify=self.ssl_verify)
             return self._request_result(res)
 
@@ -1407,7 +1242,6 @@ class SdSecureClient(_SdcCommon):
         elif len(matched_profiles) >= 2:
             return [False, matched_profiles]
 
-
     def __get_matched_profileIDs(self, requested_profile, profile_list):
         '''
         **Description**
@@ -1507,4 +1341,3 @@ class SdSecureClient(_SdcCommon):
                 matched_profiles.append(profile)
 
         return matched_profiles
-
